/*
 * Copyright (c) 1997  Dustin Sallings
 *
 * $Id: disks.c,v 1.7 1997/09/10 06:28:14 dustin Exp $
 */

#include <disk.h>

/* This is for the nifty brief, machine parsable output */
#define SHORTDISP

long fsbtoblk(long num, long fsbs, long bs)
{
    long ret;

    if( (strcmp(OS, "solaris")==0) )
    {
        ret=num;
    }
    else
    {
        ret=bsdfsbtoblk(num, fsbs, bs);
    }
    return ret;
}

struct diskret getdiskraw(struct statfs f)
{
    struct diskret ret;
    long avail;

    ret.bs=f.f_bsize;
    ret.used=fsbtoblk(f.f_blocks, f.f_bsize, BS) - fsbtoblk(f.f_bfree,
        f.f_bsize, BS);
    avail=fsbtoblk(f.f_bavail, f.f_bsize, BS)+ret.used;

    ret.total=fsbtoblk(f.f_blocks, f.f_bsize, BS);
    ret.free=fsbtoblk(f.f_bavail, f.f_bsize, BS);
    ret.percent=(double)ret.used/(double)avail*100;

    return(ret);
}

struct diskret getdisk(char *which)
{
    struct statfs f;

    statfs(which, &f);

    return(getdiskraw(f));
}

/* This is a nifty display for running daemonlike. */

void displaydisk(char *path, struct diskret d)
{

#ifdef SHORTDISP
    printf("%s;%ld;%ld;%ld;%.2f\n", path, d.total, d.used, d.free, d.percent);
#else
    printf("Filesystem mounted on %s\n", path);
    printf("Total:  %d\nUsed:   %d\nFree:   %d\nUsed:   %.2f%%\n",
        d.total, d.used, d.free, d.percent);
    printf("Native blocksize is %d\n", d.bs);
#endif

}

#ifdef INT_FSTYPES
int goodtype(int type)
{
    int i, ret=1;

    if(type == 0)
        return(ret);

    for(i=0; badfstypes[i]!=0; i++)
    {
        if(type==badfstypes[i])
           ret=0;
    }

    return(ret);
}
#else
int goodtype(char *type)
{
    int i, ret=1;

    if(type == NULL)
        return(ret);

    if(strlen(type)==0)
        return(ret);

    for(i=0; badfstypes[i]; i++)
    {
        if(strcmp(type, badfstypes[i])==0)
           ret=0;
    }

    return(ret);
}
#endif

/* here's what we do on OSen with mtab's */

#if defined(MTABFILE) && !defined(_AIX)

int domtab(int do_display)
{
    FILE *f;
    char fn[PATH_MAX];
    char *type, *path, *mtab;
    int i, total=0;

    f=fopen(MTABFILE, "r");
    if(f==NULL)
    {
        printf("Error: can't open %s\n", MTABFILE);
        return(-1);
    }

    for(;;)
    {
        fgets(fn, PATH_MAX, f);

        if(feof(f))
            break;

        for(i=0; isspace(fn[i])==0; i++); i++;
        path=fn+i;
        for(; isspace(fn[i])==0; i++);
        fn[i]=0x00;
        type=fn+i+1;
        for(i++; isspace(fn[i])==0; i++);
        fn[i]=0x00;

        if(goodtype(type))
        {
            total++;
            if(do_display==DO_DISPLAY)
                displaydisk(path, getdisk(path));
        }
    }

    fclose(f);
    return(total);
}

#else /* If the os doesn't have an mtabfile, let's check for getmntinfo */

# ifdef HAVE_GETMNTINFO

int domtab(int do_display)
{
    int i, n, total=0;
    struct statfs *mtab;

    n=getmntinfo(&mtab, 0);

    for(i=0; i<n; i++)
    {
#ifdef INT_FSTYPES
        if(goodtype(mtab[i].f_type))
#else
        if(goodtype(mtab[i].f_fstypename))
#endif
        {
            total++;
            if(do_display==DO_DISPLAY)
                displaydisk(mtab[i].f_mntonname, getdiskraw(mtab[i]));
        }
    }

    return(total);
}

# else
#ifdef _AIX

/*
 * Weird, AIX specific stuff.
 *
 * This is hackery, I don't care what it gives me, just as long as it
 * gives me a filesystem.
 */

int domtab(int do_display)
{
    FILE *f;
    char fn[PATH_MAX];
    char *type, *path, *mtab;
    int i, total=0;

    f=fopen(MTABFILE, "r");
    if(f==NULL)
    {
        printf("Error: can't open %s\n", MTABFILE);
        return(-1);
    }

    for(;;)
    {
        fgets(fn, PATH_MAX, f);

        if(feof(f))
            break;

        if(fn[0]=='/')
        {
            for(i=0; fn[i]!=':'; i++); i++;
            fn[i-1]=0x00;
            path=fn;

                total++;
                if(do_display==DO_DISPLAY)
                    displaydisk(path, getdisk(path));
        }
    }

    fclose(f);
    return(total);
}
#  else
#    error "No MTABFILE and no getmntinfo(), you're screwed."
#  endif
# endif
#endif /* MTAB section */

void showdisks(char *arg)
{
    printf("%d\n", domtab(NO_DISPLAY));
    domtab(DO_DISPLAY);
}

void showdisk(char *arg)
{
    if(arg!=NULL)
        displaydisk(arg, getdisk(arg));
    else
        puts("Must specify which disk to display.");
}
